package ping;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.Date;
import san.application.IApplication;
import san.core.IApplicationAPI;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;


public class Ping
        implements IApplication, ReceiveDataListener {

    private Object lock = new Object();
    private TransmitData received = null;

    public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
        applicationAPI.addOnSendDataListener(guid);

        PrintWriter out = applicationAPI.getStdOut();
        out.println("Ping Guid: " + applicationAPI.getId() + "args " + args);
        try {
            Guid toAddr = null;
            Guid toNet = null;
            NetIdentifier pingDst = null;
            try {
                toAddr = Guid.parseGuid(args[0]);

                toNet = Guid.parseGuid(args[1]);

                pingDst = new NetIdentifier(toAddr, toNet);
            } catch (NumberFormatException e) {
                out.print(e);
                return;
            }

            Date sendingTime = new Date();
            Date destinationTime = null;
            Date receivedTime = null;

            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(bos);

            oos.writeBoolean(false);

            long time = sendingTime.getTime();
            oos.writeLong(time);

            oos.close();

            TransmitData data = new TransmitData(bos.toByteArray());

            CodePkgIdent capsulePkgIdent = applicationAPI.getExecutedCodeIdent();

            applicationAPI.injectCapsule(capsulePkgIdent, pingDst, data);

            while (this.received == null) {
                Thread.sleep(1L);
            }

            if (this.received != null) {
                ByteArrayInputStream bais = new ByteArrayInputStream(this.received.getData());
                ObjectInputStream ois = new ObjectInputStream(bais);

                ois.readBoolean();

                sendingTime = new Date(ois.readLong());

                destinationTime = new Date(ois.readLong());

                receivedTime = new Date();

                out.println("Program Ping");
                out.println("************");
                out.println("Z lokalniho uzlu do ciloveho za: " +
                        (destinationTime.getTime() - sendingTime.getTime()) + " ms");
                out.println("Z ciloveho uzlu do lokalniho za: " +
                        (receivedTime.getTime() - destinationTime.getTime()) + " ms");
                out.println("Celkova doba mezi odeslanim a prijmem: " +
                        (receivedTime.getTime() - sendingTime.getTime()) + " ms");
                out.println();
                return;
            }
            out.println("Chyba pri prijmu dat od capsule.");
        } catch (Exception e) {
            out.println(e);
        }
    }

    public void receiveData(Guid sender, TransmitData data) {
        this.received = data;
        System.out.println("---------------------------------------------------------------------");
        System.out.println("---------------------D A T A   P R I S L Y---------------------------");
        System.out.println("---------------------------------------------------------------------");
    }
}