/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package randomrun;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.Date;
import java.util.Random;

import org.apache.log4j.Logger;

import san.application.IApplication;
import san.core.IApplicationAPI;
import san.core.Kernel;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;

/**
 * Runs two applications in random order for specified number of times.
 * 
 * App's parameters:
 *  1. integer - number of runs of the apps
 *  2. string - name of first app
 *  3. string - name of second app
 *  4. time between two run of an app
 *  5. list of numbers of net. interfaces (in the same network, space-separated) used as 
 *     parameters for the two apps (one random net. interface each time an app is run)
 */
public class RandomRun implements IApplication {
	
	private static final transient Logger log = Logger.getLogger(RandomRun.class);	
	
	public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
		
		PrintWriter out = applicationAPI.getStdOut();
		
		try{
			
			//read app's parameters
			int i = 0;
			int repeat = Integer.parseInt(args[0]);
			String app1 = args[1];
			String app2 = args[2];
			int pause = Integer.parseInt(args[3]);
			
			Random r = new Random((new Date()).getTime());
			int x = r.nextInt(100);
			Thread.sleep(1000 + x*10);
			
			//run apps
			for(i = 0; i < repeat; i++){
				x = r.nextInt(args.length - 4);
				x += 4; 
				String app = (x % 2 == 0) ? app2 : app1;	//choose one
				if (log.isDebugEnabled()) {
					log.debug("RUN " + app + " " + args[x] + " 1");
				}
				applicationAPI.runApplication(app, new String[]{args[x], "1"});
				Thread.sleep(((x % 2) + 1) * pause);
			}
		
		} catch(Exception e) {
				out.println(e);
		}
	}
}
