package shell;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;

import san.application.IApplication;
import san.core.AplStoreException;
import san.core.ExecuteException;
import san.core.IApplicationAPI;
import san.core.IServerControl;
import san.ident.Guid;

class ParseException extends Exception {
	
	public ParseException(String exception) {
		super(exception);
	}
}

class Command {
	
	private String commandName;
	private String[] attributes;
	
	public Command() {
		commandName="";
		attributes=new String[0];
	}
	
	/**
	 * @return the commandName
	 */
	public String getCommandName() {
		return commandName;
	}
	
	/**
	 * @param commandName the commandName to set
	 */
	public void setCommandName(String commandName) {
		this.commandName = commandName;
	}
	
	/**
	 * @return the attributes
	 */
	public String[] getAttributes() {
		return attributes;
	}
	
	/**
	 * @param attributes the attributes to set
	 */
	public void setAttributes(String[] attributes) {
		this.attributes = attributes;
	}
}


class OneCommandParser {
	
	private String inputTextStr;
	private Command command;
	
	public OneCommandParser(String inputText) {
		this.inputTextStr=inputText;
		this.command=new Command();
	}	
	
	private ArrayList string2ArrayList(String inputText) throws ParseException {
		
		if (inputText.equals("")){
				this.command=new Command();
			return new ArrayList();
		}
				
		ArrayList blocks=new ArrayList();
		
		int an=0;
		for (int i=0;i<inputText.length();i++){//syntax test
			if (inputText.charAt(i)=='"') an++;
		}
		if ((an%2)==1)
				throw new ParseException("Missed '\"'");//throws exception when the " is missing
		
		int blockNum=0;
		String actualBlock="";
		int i=0;
		int j=inputText.length();
		
		
		while (inputText.charAt(j-1)==' ') j--;
		inputText=inputText.substring(0,j);
		while (inputText.charAt(i)==' ') i++;//skip start space
		for (;i<inputText.length();i++){
			
			if (inputText.charAt(i)==' '){
				while (inputText.charAt(i)==' ') i++;
				if (!actualBlock.equals(""))
							blocks.add(actualBlock);
				actualBlock="";
			}
			
			if (inputText.charAt(i)=='"'){
				if (!actualBlock.equals("")){
					blocks.add(actualBlock);
					actualBlock="";
				}
							
				i++;
				while (inputText.charAt(i)!='"'){
					actualBlock+=inputText.charAt(i);
					i++;
				}
				i++;
				blocks.add(actualBlock);
				actualBlock="";
			}
			if	((i<inputText.length())&&(inputText.charAt(i)!=' '))					
				actualBlock+=inputText.charAt(i);
		}
		
		if (!actualBlock.equals(""))
						blocks.add(actualBlock);
		return blocks;
	}
	
	public void parse() throws ParseException {
		ArrayList blocks = string2ArrayList(inputTextStr);
		if (blocks.size()==0)
			return;
		this.command.setCommandName((String) blocks.get(0));
		String[] attribs=new String[0];
		if (blocks.size()> 1){
			attribs=new String[blocks.size()-1];
			for (int k=1;k<blocks.size();k++)
				attribs[k-1]=(String)blocks.get(k);
		}
		this.command.setAttributes(attribs);		
	}
	
	public Command getCommand(){
		return this.command;
	}
}


public class Shell extends Thread implements IApplication {

	private IApplicationAPI applicationAPI;
	private IServerControl serverControl;
	private BufferedReader input;
	private PrintWriter output;
	
	private boolean running = true;
	
	public Shell() {
		//default constructor is not used
	}
	
	public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
		this.applicationAPI = applicationAPI;
		this.serverControl = applicationAPI.getServerControl();
		this.input = new BufferedReader(new InputStreamReader(applicationAPI.getStdIn()));
		this.output = applicationAPI.getStdOut();
		try{
			while(running) {
				output.print("> ");
				executeCommand(input.readLine());
			}
		} catch(IOException e) {
			System.err.println("Error reding console input.");
		}
	}

	private void executeCommand(String readSeq) {
		if(readSeq == null) {
			return;	
		}
		
		int enterIndex = readSeq.indexOf("\n");	//delete enter character
		if(enterIndex > 0)
			readSeq = readSeq.substring(0,enterIndex);
		
		if(readSeq.length() == 0)
			return;
		
		if(readSeq.charAt(0)=='\n')
			return;
		
		OneCommandParser p =new OneCommandParser(readSeq);
		try{
			p.parse();
			runOneCommand(p.getCommand());
		}catch(ParseException e) {
			output.print("Error in parsing: " + e.getMessage()+"\n");
		}catch(ExecuteException e) {
			output.print("Error in executing: " + e.getMessage()+"\n");
		}catch(AplStoreException e) {
			output.print("Error in storing: " + e.getMessage()+"\n");
		}
	}
	
	/**
	 * Execute one command.
	 */
	private void runOneCommand(Command command) throws ExecuteException, AplStoreException {
		String[] att = command.getAttributes();
		if(command.getCommandName().compareTo("echo") == 0){
			for (int i = 0; i < att.length; i++) {
				output.print(att[i]+" ");
			}
		} else if(command.getCommandName().compareTo("exit") == 0) {
			serverControl.shutDownServer();
		} else if(command.getCommandName().compareTo("store") == 0){
			if(att.length == 1){
				String ident = serverControl.storeApplication(att[0]);
				output.print("Application was stored with identificator:\n"+ident+"\n");
			}
			else
				throw new ExecuteException("An invalid argument of the store command.");
		} else if(command.getCommandName().compareTo("run") == 0) {
			if(att.length >= 1){
				String[] att2 = new String[att.length-1];
				for (int i = 0; i < att2.length; i++) {
					att2[i] = att[i+1];
				}
				try{
					Guid appGuid = serverControl.runApplication(att[0], att2,applicationAPI.getId());
          output.print("app guid " + appGuid);
					applicationAPI.waitForApplication(appGuid);
				}catch(InterruptedException e) {
					//
				}
			}
			else
				throw new ExecuteException("To few for the command.");
		}
		else {
			throw new ExecuteException("This command is not supported: " + command.getCommandName());
		}
	}
}