/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.codeDistribution;

import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

import san.core.Kernel;
import san.ident.NetIdentifier;

/*
 * Its purpose is to collect informations about one application
 */
public class AppPlacementInfo {
	
	private boolean storedLocaly = false;	//is in repository
	
	//identifiers of nodes where a capsule with this application came from
	private Set<NetIdentifier> nodesAppCameFrom = Collections.synchronizedSet(new LinkedHashSet<NetIdentifier>());	
	
	//identifiers of nodes where a capsule with data for this application came from
	private Set<NetIdentifier> nodesDataCameFrom = Collections.synchronizedSet(new LinkedHashSet<NetIdentifier>());
	
	//identifiers of nodes where a capsule with this application was sent to
	private Set<NetIdentifier> nodesAppSentTo = Collections.synchronizedSet(new LinkedHashSet<NetIdentifier>());
	
	//identifiers of nodes where a capsule with data for this application was sent to
	private Set<NetIdentifier> nodesDataSentTo = Collections.synchronizedSet(new LinkedHashSet<NetIdentifier>());

	private Kernel kernel;	
	
	public AppPlacementInfo(Kernel kernel) {
		super();
		this.kernel = kernel;
	}

	public void addNodeAppCameFrom(NetIdentifier i) {
		if(kernel.isMyInterfaceOrNull(i)) return;			
		nodesAppCameFrom.add(i);
	}	

	public void addNodeDataCameFrom(NetIdentifier i) {
		if(kernel.isMyInterfaceOrNull(i)) return;
		nodesDataCameFrom.add(i);
	}

	public void addNodeAppSentTo(NetIdentifier i) {
		if(kernel.isMyInterfaceOrNull(i)) return;
		nodesAppSentTo.add(i);
	}

	public void addNodeDataSentTo(NetIdentifier i) {
		if(kernel.isMyInterfaceOrNull(i)) return;
		nodesDataSentTo.add(i);
	}

	public void removeNodeAppCameFrom(NetIdentifier i) {
		nodesAppCameFrom.remove(i);
	}

	public void removeNodeDataCameFrom(NetIdentifier i) {
		nodesDataCameFrom.remove(i);
	}

	public void removeNodeAppSentTo(NetIdentifier i) {
		nodesAppSentTo.remove(i);
	}

	public void removeNodeDataSentTo(NetIdentifier i) {
		nodesDataSentTo.remove(i);
	}

	public boolean isStoredLocaly() {
		return storedLocaly;
	}

	public void setStoredLocaly(boolean storedLocaly) {
		this.storedLocaly = storedLocaly;
	}

	public Set<NetIdentifier> getNodesAppCameFrom() {
		return nodesAppCameFrom;
	}

	public Set<NetIdentifier> getNodesDataCameFrom() {
		return nodesDataCameFrom;
	}

	public Set<NetIdentifier> getNodesAppSentTo() {
		return nodesAppSentTo;
	}

	public Set<NetIdentifier> getNodesDataSentTo() {
		return nodesDataSentTo;
	}
}
