/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.ii.InterceptorInjector;
import san.ii.Packet;
import san.softstate.ISoftState;

/**
 * <b>Title:</b> Smart Active Node<br> <b>Description:</b> The API witch is used by application process.<br>
 * @author     Michal Rejda
 * @since     21.10.2007
 * @version     1.0
 * @uml.dependency   supplier="san.core.IServerControl"
 * @uml.dependency   supplier="san.softstate.ISoftState"
 */
public interface IApplicationAPI {

	/**
	 * Create new capsule. This capsule is afterwards send to the destination.
	 * @throws DestinationNotAvailableException if destination is not jet available
	 */
	public void injectCapsule(CodePkgIdent executedCode,
			NetIdentifier destination, TransmitData payload) throws DestinationNotAvailableException;
	
	public void injectCapsule(CodePkgIdent executedCode,
			NetIdentifier destination, TransmitData payload, boolean executeContinuously) throws DestinationNotAvailableException;

	/**
	 * Return server control API.
	 */
	public IServerControl getServerControl();

		/**
	 * All IOnSendDataListener to listen onSendData event.
	 */
	public void addOnSendDataListener(Guid listener);

	/**
	 * Remove listener. Return true if listener was existed.
	 */
	public boolean removeOnSendDataListener(ReceiveDataListener listener);

	/**
	 * Run application with specific code identification.
	 */
	public Guid runApplication(CodePkgIdent ident, String[] args)
			throws ExecuteException;

	/**
	 * Run application with specific name.
	 */
	public Guid runApplication(String name, String[] args)
			throws ExecuteException;

	/**
	 * Wait for end of application.
	 * @param applId application id
	 */
	public void waitForApplication(Guid applId) throws InterruptedException;

	/**
	 * Wait for end of collection of application.
	 * @param applId application id
	 */
	public void waitForApplication(Collection<Guid> applId)
			throws InterruptedException;

	/**
	 * Return standard input stream.
	 */
	public InputStream getStdIn();

	/**
	 * Return standard output writer.
	 */
	public PrintWriter getStdOut();

	/**
	 * Return application id.
	 */
	public Guid getId();
	
	/**
	 * Return identification of executed code.
	 */
	public CodePkgIdent getExecutedCodeIdent();
	
	/**
	 * Return soft state.
	 */
	public ISoftState getSoftState();
	
	/**
	 * TODO user authentication 
	 *  i think user adding a user is done 
	 */
	
	/**
	 * check the file integration with MAC
	 */
	public boolean startNodeAuth(String key);
	
	/**
	 * check the login of a desired username
	 * @param id
	 * @param passwd
	 * @return
	 */
	public boolean checkLogin(String id, String passwd);
	
	/**
	 * get the users avaiables from the file
	 * @param key
	 * @return
	 */
	public List<String> getUsers(String key);
	
	/**
	 * get the arguments for adding a user
	 * @param lista
	 * @param nome
	 * @param pwd
	 */
	public void getArgs(LinkedList<String> lista, String nome, String pwd);
	
	/**
	 * writes to the file the old users and the new ones
	 * @param lista
	 * @param key
	 */
	public void writePwds(List<String> lista,String key);
	
	public String getNodeInfo();

  /**
   * This method makes sending from appliaction data to server possible
   */
  public void sendPayload(Packet p);
}
