/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.ii.InterceptorInjector;
import san.ii.Packet;
import san.routing.IRoutingTableControl;
import san.softstate.ISoftState;

/**
 * <b>Title:</b> Smart Active Node<br> <b>Description:</b> The API witch is used by capsule process.<br>
 * @author       Michal Rejda
 * @since       21.10.2007
 * @version       1.0
 * @uml.dependency   supplier="san.routing.IRoutingTableControl"
 * @uml.dependency   supplier="san.core.IServerControl"
 * @uml.dependency   supplier="san.softstate.ISoftState"
 */
public interface ICapsuleAPI {

	/**
	 * Set capsule destination.
	 */
	public abstract void setDestination(NetIdentifier dest);

	public abstract void setSource(NetIdentifier i);
	
	/**
	 * Return data load of the capsule.
	 */
	public abstract TransmitData getDataLoad();

	/**
	 * Return capsule source node.
	 */
	public abstract NetIdentifier getSource();

	/**
	 * Send data from the capsule to the process.
	 */
	public abstract void sendData(Guid toProcess, TransmitData data);

	/**
	 * This method send data to application on local node. For identification of
	 * the application is used it's Code Package Identifier. Data are send to
	 * multiple application if on the server are some instances of application
	 * running on specific code.
	 * @param processCode Code Package Identifier of the application.
	 * @param data data load
	 */
	public abstract void sendData(CodePkgIdent processCode, TransmitData data);

	/**
	 * Return true if concrete network identifier exist on this server.
	 */
	public abstract boolean isThisServer(NetIdentifier ident);

	/**
	 * Return capsule destination.
	 */
	public abstract NetIdentifier getDestination();

	/**
	 * Return source application ID. This ID is assigned by kernel. It isn't the
	 * code package identification!
	 */
	public abstract Guid getSourceApplication();

	/**
	 * Set data load.
	 */
	public abstract void setDataLoad(TransmitData data);

	/**
	 * Capsule finished. It doesn't want to be send.
	 */
	public abstract void finished();

	/**
	 * Return soft state.
	 */
	public abstract ISoftState getSoftState();
	
	/**
	 * Return object that provides to make changes in routing table.
	 */
	public IRoutingTableControl getRoutingModification();
	
	/**
	 * Create new capsule. This capsule is afterwards send to the destination.
	 * @throws DestinationNotAvailableException if destination is not jet available
	 */
	public void injectCapsule(CodePkgIdent executedCode,
			NetIdentifier destination, TransmitData payload) throws DestinationNotAvailableException;
	
	/**
	 * Return identification of executed code.
	 */
	public CodePkgIdent getExecutedCodeIdent();
	
	public void setExecuteContinuously(boolean b);
	
	public NetIdentifier getPrev();
	
	public NetIdentifier getOver();

  /**
   * This method makes sending from application data to server possible
   */
  public void sendPayload(Packet p);
}
