/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.LinkedList;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import san.applicationRepository.SanCodePkg;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.ii.InterceptorInjector;
import san.ii.Packet;
import san.interpreter.sanint.classLoading.ClassManager;
import san.interpreter.sanint.classLoading.Reference;
import san.networkInterfaces.NetInterface;
import san.security.userSecurity.AddUser;
import san.security.userSecurity.UserSecurity;
import san.softstate.ISoftState;
import java.util.logging.*;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class is an application process.<br>
 * @author Michal Rejda
 * @since 21.10.2007
 * @version 1.0
 */
public class ProcessApplication extends Process implements IApplicationAPI {

	private static final transient Logger log = Logger.getLogger(ProcessApplication.class.getName());
	
		/**
	 * @uml.property name="onSendDataListeners"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="processApplication:san.data.OnSendDataListener"
	 */
	private Collection<Guid> onSendDataListeners;
	/**
	 * @uml.property name="args" multiplicity="(0 -1)" dimension="1"
	 */
	private String[] args;

	public ProcessApplication(Kernel kernel) {
		super(kernel);
		this.onSendDataListeners = new LinkedList<Guid>();
	}

	public void addOnSendDataListener(Guid listener) {
		synchronized (onSendDataListeners) {
			onSendDataListeners.add(listener);
		}
	}

	public boolean removeOnSendDataListener(ReceiveDataListener listener) {
		synchronized (onSendDataListeners) {
			return onSendDataListeners.remove(listener);
		}
	}

	protected void fireOnSendDataListeners(Guid sender, TransmitData data) {
		synchronized (onSendDataListeners) {
			for (Guid listener : onSendDataListeners) {
				try
				{
					Reference thisRef = kernel.getInterpretManager().getRunningApplication(listener);
					ClassManager cman = new ClassManager((SanCodePkg)this.codePackage);
					//receiveData(Guid sender, TransmitData data);
					cman.runMethod(thisRef.getClassInfo().getName(), "receiveData", 
							"(Lsan/ident/Guid;Lsan/data/TransmitData;)V", 
							new Object[] {
								sender,
								data,
							}, thisRef, false);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
				///listener.receiveData(sender, data);
			}
		}
	}


	public void injectCapsule(CodePkgIdent executedCode,
			NetIdentifier destination, TransmitData payload)
			throws DestinationNotAvailableException {
		getKernel().injectCapsule(executedCode, getId(), destination, payload);
	}
	
	@Override
	public void injectCapsule(CodePkgIdent executedCode,
			NetIdentifier destination, TransmitData payload,
			boolean executeContinuously)
			throws DestinationNotAvailableException {

		getKernel().injectCapsule(executedCode, getId(), destination, payload, executeContinuously);
		
	}

	public IServerControl getServerControl() {
		return getKernel();
	}

	@Override
	public Guid getId() {
		return super.getId();
	}

	public CodePkgIdent getExecutedCodeIdent() {
		return codePackage.getPkgIdentification();
	}

	public InputStream getStdIn() {
		if (console != null)
			return console.getIn();
		return null;
	}

	public PrintWriter getStdOut() {
		if (console != null)
			return console.getOut();
		return null;
	}

	public Guid runApplication(CodePkgIdent ident, String[] args)
			throws ExecuteException {
		return kernel.runApplication(ident, args, id);
	}

	public Guid runApplication(String name, String[] args)
			throws ExecuteException {
		return kernel.runApplication(name, args, id);
	}

	public void waitForApplication(Collection<Guid> applId)
			throws InterruptedException {
		kernel.waitForApplication(id, applId);
	}

	public void waitForApplication(Guid applId) throws InterruptedException {
		Collection<Guid> waitFor = new Vector<Guid>();
		waitFor.add(applId);
		waitForApplication(waitFor);
	}

	/**
	 * Getter of the property <tt>args</tt>
	 * @return Returns the args.
	 * @uml.property name="args"
	 */
	public String[] getArgs() {
		return args;
	}

	/**
	 * Setter of the property <tt>args</tt>
	 * @param args The args to set.
	 * @uml.property name="args"
	 */
	public void setArgs(String[] args) {
		this.args = args;
	}
	
	public ISoftState getSoftState() {
		return kernel.getSoftState();
	}
	

	public boolean addUser(String masterPass, String newUserName, String pass) {
		return addUser(masterPass, newUserName, pass);
	}

	public boolean startNodeAuth(String key) {
		return UserSecurity.startNodeAuth(key);
	}

	public boolean checkLogin(String id, String passwd) {
		return UserSecurity.checkLogin(id, passwd);
	}

	public List<String> getUsers(String key) {
		return AddUser.getUsers(key);
	}

	public void getArgs(LinkedList<String> lista, String nome, String pwd) {
		AddUser.getArgs(lista, nome, pwd);
	}

	public void writePwds(List<String> lista, String key) {
		AddUser.writePwds(lista, key);
	}

	@Override
	public String getNodeInfo() {	
		String info = "";
		
		for(NetIdentifier n : kernel.getNetInterfaces().getNetInterfaces().keySet()){			
			info += "Local interface: \n";
			info += "address: " + n.getAddress().toString() + "\n";
			info += "network: " + n.getNetwork().toString() + "\n";
			NetInterface ni = kernel.getNetInterfaces().getNetInterfaces().get(n);
			for(NetIdentifier r : ni.getRemoteInterfaces().keySet()){
				info += "\t\tRemote interface: \n";
				info += "\t\taddress: " + r.getAddress().toString() + "\n";
				info += "\t\tnetwork: " + r.getNetwork().toString() + "\n";
			}						
		}	
		
		info += "Applications: \n";
		for(String s : kernel.getRepository().getCpBaseOnPkgNamesMapKeys()){
			info += "app: " + s + "\n";
		}
		
		return info;
	}

  public void sendPayload(Packet p) {
    try {
      InterceptorInjector.getInterceptorInjector().sendPayload(p);
    } catch (Exception ex) {
      // Just do nothing - no data will be delivered
    }
  }
}
