/*************************************************************************
 *                           Smart Active Node                           * 
 *                      copyright (C) 2007 KIV-FAV-ZCU                   *
 *************************************************************************
 * The licence for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.interpreter;

import san.applicationRepository.BshCodePkg;
import san.applicationRepository.CodePackage;
import san.applicationRepository.SanCodePkg;
import san.core.IScheduler;
import san.ident.Guid;
import san.interpreter.BSH.BshInterpreter;
import san.interpreter.sanint.classLoading.HardClass;
import san.interpreter.sanint.classLoading.Reference;
import san.interpreter.sanint.interpreter.SanInterpreter;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import san.core.Kernel;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> Manage interpreters. Create interpreters for specific
 * code packages.<br>
 * @author Michal Rejda
 * @since 20.10.2007
 * @version 1.0
 */
public class InterpretManager {

	/**
	 * @uml.property name="kernel"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="interpretManager:san.core.Kernel"
	 */
	private Kernel kernel = null;
	/**
	 * @uml.property name="scheduler"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="interpretManager:san.core.Scheduler"
	 */
	private IScheduler scheduler = null;
	/**
	 * @uml.property name="interpreters"
	 * @uml.associationEnd readOnly="true" aggregation="shared"
	 *                     inverse="interpretManager:san.interpreter.IInterpreter"
	 *                     qualifier="interpretType:java.lang.String
	 *                     san.interpreter.IInterpreter"
	 */
	private Map<String, IInterpreter> interpreters;

	private Map<Guid, Reference> runningApplications;
	
	public InterpretManager(Kernel kernel) {
		this.kernel = kernel;
		this.interpreters = new TreeMap<String, IInterpreter>();
		this.runningApplications = new HashMap<Guid, Reference>();
	}

	public void addRunningApplication(Guid guid, Reference reference)
	{
		runningApplications.put(guid, reference);
	}
	public void removeRunningApplication(Guid guid)
	{
		runningApplications.remove(guid);
	}
	public Reference getRunningApplication(Guid guid)
	{
		return runningApplications.get(guid);
	}
	
	/**
	 * Return code interpreter for specific code.
	 */
	public IInterpreter getInterpreter(CodePackage code) {
		if (code instanceof BshCodePkg) {
			BshInterpreter bsh = new BshInterpreter(this, kernel);
			return bsh;
		} else if(code instanceof SanCodePkg) {
			SanInterpreter si = new SanInterpreter(this, kernel);
			return si;
		}
		return this.interpreters.get(code.getClass().getName());
	}

	/**
	 * Indicates that the interpreter hasn't more for interpreting. The program
	 * has ended.
	 */
	public void interpretingEnded(IInterpreter interpreter) {
		this.scheduler.intepretingEnded(interpreter.getProcess());
	}

	/**
	 * Getter of the property <tt>scheduler</tt>
	 * @return Returns the scheduler.
	 * @uml.property name="scheduler"
	 */
	public IScheduler getScheduler() {
		return scheduler;
	}

	/**
	 * Setter of the property <tt>scheduler</tt>
	 * @param scheduler The scheduler to set.
	 * @uml.property name="scheduler"
	 */
	public void setScheduler(IScheduler scheduler) {
		this.scheduler = scheduler;
	}
}
