package san.interpreter.sanint.classLoading;
import san.interpreter.sanint.interpreter.References;
import san.interpreter.sanint.interpreter.InterpreterStack;

import java.nio.ByteBuffer;
import java.util.HashMap;

import com.sun.org.apache.bcel.internal.classfile.ConstantClass;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;


public class HardReference extends Reference {
	ByteBuffer data;
	HardClass classInfo;
	Reference superClass;
	Reference[] references;
	
	
	public HardReference(ClassManager cm, HardClass classInfo)
	{
		data = ByteBuffer.allocate(Math.max(1,classInfo.getDataLength()));
		this.classInfo = classInfo;
		references = new Reference[classInfo.getNumReferences()];
		for(int i = 0; i < references.length; i++)
			references[i] = new SoftReference(null);
		if (classInfo.getSuperClass() != null)
			superClass = cm.createReference(classInfo.getSuperClass());
	}

	public int  getField4B(int index)
	{
		int i = classInfo.getVariableOffset(index);
		return data.getInt(i);
	}	
	public long getField8B(int index)
	{
		int i = classInfo.getVariableOffset(index);
		return data.getLong(i);
	}
	public Reference getReference(int index)
	{
		int i = classInfo.getVariableOffset(index);
		return references[i];
	}
	
	public void setField4B(int index, int value)
	{
		int i = classInfo.getVariableOffset(index);
		data.putInt(i, value);
	}
	public void setField8B(int index, long value)
	{
		int i = classInfo.getVariableOffset(index);
		data.putLong(i, value);
	}
	public void setReference(int index, HardReference value)
	{
		int i = classInfo.getVariableOffset(index);
		references[i] = value;
	}
	
	
	public HardClass getClassInfo()
	{
		return classInfo;
	}
	@Override
	public String toString()
	{
		return classInfo.getName();
	}

	@Override
	public boolean checkCast(Class cl, int index) {
		// TODO Auto-generated method stub
		return false;
	}

	public void	getVariable(HardClass currentClass, int index, InterpreterStack stack, References refs)
		throws Exception
	{
		int length = currentClass.getVariableLength(index);
		getVariable(currentClass.getVariableName(index), length, stack, refs);
	}
	public void getVariable(String name, int length, InterpreterStack stack, References refs)
		throws Exception
	{
		int i = classInfo.getVariableOffset(name);
		if (superClass == null)
			System.out.println();
		if (i < 0)
			superClass.getVariable(name, length, stack, refs);
		else getVariable(i, length, stack, refs);	
	}
	public Object getVariable(String name)
		throws Exception
	{
		VariableInfo info = classInfo.getVariableInfos().get(name);
		switch(info.getType())
		{
			case 'I': //integer
				return new Integer(data.getInt(info.getOffset()));
			case 'F': //float
				return new Float(data.getFloat(info.getOffset()));
			case 'D': //double
				return new Double(data.getDouble(info.getOffset()));
			case 'J': //long
				return new Long(data.getLong(info.getOffset()));
			case 'Z': //boolean			
				byte byteData = data.get(info.getOffset());
				if (byteData == 0)
					return Boolean.FALSE;
				return Boolean.TRUE;
			case 'B': //byte
				return new Byte(data.get(info.getOffset()));
			case 'C': //char
				return new Character(data.getChar(info.getOffset()));
			case 'S': //short
				return new Short(data.getShort(info.getOffset()));
			case '[': //array
			case 'L': //reference
				return references[info.getOffset()];
		}
		throw new Exception();
	}
	protected void getVariable(int offset, int length, InterpreterStack stack, References refs)
		throws Exception
	{
		switch (length)
		{
			case 1:  stack.write(data.get(offset)); return;
			case 2:  stack.write(data.getShort(offset)); return;
			case 4:  stack.write(data.getInt(offset)); return;
			case 8:  stack.write(data.getLong(offset)); return;
			case -4: stack.write(refs.putReference(references[offset])); return;
		}
		throw new Exception("Error: getVariable - variable with length " + length + 
									" not recognized.");
	}

	@Override
	public void setVariable(HardClass currentClass, int index, InterpreterStack stack, References refs, long value) 
		throws Exception
	{
		String name = currentClass.getVariableName(index);
		int length = currentClass.getVariableLength(index);
		setVariable(name, length, stack, refs, value);
	}
	public void setVariable(String name, int length, InterpreterStack stack, References refs, long value)
		throws Exception
	{
		int i = classInfo.getVariableOffset(name);
		if (i < 0)
			superClass.setVariable(name, length, stack, refs, value);
		else setVariable(i, length, stack, refs, value);
	}
	
	public void setVariable(int offset, int length, InterpreterStack stack, References refs, long value)
		throws Exception
	{
		switch (length)
		{
			case 1: //byte dataByte = stack.readByte();
					data.put(offset, (byte)value);
					return;
			case 2: //short dataShort= stack.readShort();
					data.putShort(offset, (short)value);
					return;
			case 4:	//int dataInt = stack.readInt();
					data.putInt(offset, (int)value);
					return;
			case 8: //long dataLong = stack.readLong();
					data.putLong(offset, value);
					return;
			case -4:// int refIndex = stack.readInt();
					references[offset] = refs.getReference((int)value);
					return;
		}
		throw new Exception("Error: setVariable - variable with length " + length + 
									" not recognized.");
	}
	public boolean isNull()
	{
		return data == null;
	}

	@Override
	public Reference getSuperReference() 
		throws Exception
	{
		return superClass;
	}

	@Override
	public void fromObject(HashMap<Long, Object> convertedObjects,
			Object newData) throws Exception {
		//throw new NotImplementedException();
		
	}

	@Override
	public boolean isInstanceOf(HardClass currentClass, int classIndex)
			throws Exception {
		ConstantClass clConst =  (ConstantClass)currentClass.getConstants().getConstant(classIndex);
		String className = clConst.getBytes(currentClass.getConstants());
		return classInfo.canBeCasted(className);
	}
}
