package san.interpreter.sanint.classLoading;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

public class ReferenceConverter {

	private static final transient Logger log = Logger.getLogger(ReferenceConverter.class);
	
	public Object toJavaObject(Reference ref, HashMap<Long, Object> convertedObjects)
		throws Exception
	{
		if (ref == null || ref.isNull())
			return null;
		
		if (ref instanceof SoftReference)
			return ((SoftReference)ref).getReference();
		
		if (convertedObjects == null)
			convertedObjects = new HashMap<Long, Object>();
		
		if (ref instanceof Array)
			return ((Array)ref).toArray(convertedObjects);
	
		HardReference href = (HardReference)ref;
		HardClass cl = href.getClassInfo();
    java.lang.Class objCl;
    //try {
      objCl = java.lang.Class.forName(cl.getName().replace('/', '.'));
    //} catch (ClassNotFoundException cnfe) {
       /*TODO: Write own class loader that tries to locate classes inside application packages */
    //}
		Object ret = createInstance(objCl);
	
		convertedObjects.put(ref.getInstanceId(), ret);
		
		while(cl != null)
		{
			objCl = java.lang.Class.forName(cl.getName().replace('/', '.'));
			Map<String, VariableInfo> vars = cl.getVariableInfos();
			for(String name : vars.keySet())
			{
				Object obj = href.getVariable(name);
				if (obj instanceof Reference)
				{
					if (convertedObjects.containsKey(((Reference) obj).getInstanceId()))
						obj = convertedObjects.get(((Reference) obj).getInstanceId());
					else obj = toJavaObject((Reference)obj, convertedObjects);
				}
				Field varField = objCl.getDeclaredField(name);
				varField.setAccessible(true);
				try
				{
					varField.set(ret, obj);
				}
				catch (Exception e)
				{
					System.out.println();
				}
			}
			if (cl.getSuperClass() instanceof SoftClass)
				throw new NotImplementedException();
			cl = (HardClass)cl.getSuperClass();
			href = (HardReference)href.superClass;
		}
		
		return ret;
	}
	public Reference toHardReference(Object obj, ClassManager manager)
		throws Exception
	{
		return new SoftReference(obj, manager);
		/*if (obj == null)
			return new SoftReference(null, manager);
		
		if (obj instanceof java.lang.reflect.Array)
			throw new NotImplementedException();
		
		Class cl = manager.getClass(obj.getClass().getName());
		if (cl instanceof SoftClass)
			return new SoftReference(obj, manager);
		HardClass hcl = (HardClass)cl;
		HardReference ret = (HardReference)manager.createReference(hcl);
		
		while(cl != null)
		{
		
		}
		
		throw new NotImplementedException();*/
	}
	
	
	private Object createInstance(java.lang.Class cl)
		throws Exception
	{
		
		try
		{
			return cl.newInstance();
		} 
		catch (Exception e) {}
		
		if (cl.getName().equals("java.net.URI"))
			System.out.println();
			
		try
		{
			java.lang.Class specialObjectCreator = java.lang.Class.forName("san.interpreter.sanint.classLoading.SpecialObjectCreator");
			Method specialCreationMethod = specialObjectCreator.getMethod("create" + cl.getName().replace('.', '_'), null);
			return specialCreationMethod.invoke(null, null);
		}catch (NoSuchMethodException e) {}

		
		
		Constructor constructor = null;
		int minArgsCount = Integer.MAX_VALUE;
		for(Constructor c : cl.getDeclaredConstructors())
		{
			if (c.getParameterTypes().length < minArgsCount)
			{
				constructor = c;
				minArgsCount = c.getParameterTypes().length;
			}
		}
    if (constructor == null) {
      return null;
    }
		constructor.setAccessible(true);
		java.lang.Class[] types = constructor.getParameterTypes();
		Object[] params = new Object[types.length];
		for(int i = 0; i < types.length; i++)
		{
			try
			{
				if (types[i].isPrimitive())
				{
					if (int.class.equals(types[i]))
						params[i] = new Integer(0);
					else if (float.class.equals(types[i]))
						params[i] = new Float(0);
					else if (long.class.equals(types[i]))
						params[i] = new Long(0);
					else if (double.class.equals(types[i]))
						params[i] = new Double(0);
					else if (short.class.equals(types[i]))
						params[i] = new Short((short)0);
					else if (char.class.equals(types[i]))
						params[i] = new Character((char)0);
					else if (byte.class.equals(types[i]))
						params[i] = new Float(0);
					else if (boolean.class.equals(types[i]))
						params[i] = Boolean.TRUE;
				}
				else if(types[i].isInterface() || types[i].isEnum() ||
						types[i].equals(cl))
				{
					params[i] = null;
				}
				else
				{
					params[i] = createInstance(types[i]);
				}
			}
			catch(Exception e) 
			{
				e.printStackTrace();
			}
		}
		try {
		return constructor.newInstance(params);
		} catch (Exception e)
		{
			log.error("Nepodarilo se vytvorit instanci tridy " + cl.getName());
			//System.out.println("Nepodarilo se vytvorit instanci tridy " + cl.getName());
			//e.printStackTrace();
			log.error(e);
		}
		return null;
	}
}
