package san.interpreter.sanint.classLoading;

import java.nio.ByteBuffer;
import java.util.HashMap;

import org.apache.log4j.Logger;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.interpreter.References;

public class SimpleArray extends Array {
	
	private static final transient Logger log = Logger.getLogger(SimpleArray.class);
	
	public final int ACODE_BOOLEAN 	= 4;
	public final int ACODE_CHAR 	= 5;
	public final int ACODE_FLOAT 	= 6;
	public final int ACODE_DOUBLE 	= 7;
	public final int ACODE_BYTE 	= 8;
	public final int ACODE_SHORT 	= 9;
	public final int ACODE_INT 		= 10;
	public final int ACODE_LONG 	= 11;	
	
	private int size;
	ByteBuffer data;
	private int length;
	private int aCode;
	
	public SimpleArray(int count, int aCode)
		throws Exception
	{
		this.aCode = aCode;
		switch(aCode)
		{
			case ACODE_BYTE:
			case ACODE_BOOLEAN: size = 1; break;
			case ACODE_SHORT:	  
			case ACODE_CHAR: size = 2; break;
			case ACODE_FLOAT: 
			case ACODE_INT: size = 4; break;
			case ACODE_DOUBLE:
			case ACODE_LONG: size = 8; break;
			default: throw new Exception("Error: SimpleArray - invalid array type");
		}
		length = count;
		data = ByteBuffer.allocate(count * size);
	}
	
	public SimpleArray(Object newArr)
	{
		if (newArr == null) {
			data = null;
			return;
		}
		length = java.lang.reflect.Array.getLength(newArr);
		if (newArr.getClass().equals(int[].class)) {
			aCode = ACODE_INT;
			size = 4;
			data = ByteBuffer.allocate(length * size);
			for(int i = 0; i < length; i++)
				data.putInt(i * size, java.lang.reflect.Array.getInt(newArr, i));
		}
		else if (newArr.getClass().equals(byte[].class)) {
			aCode = ACODE_BYTE;
			size = 1;
			data = ByteBuffer.allocate(length);
			for(int i = 0; i < length; i++)
				data.put(i, java.lang.reflect.Array.getByte(newArr, i));
		}
		else if (newArr.getClass().equals(float[].class)) {
			aCode = ACODE_FLOAT;
			size = 4;
			data = ByteBuffer.allocate(length * size);
			for(int i = 0; i < length; i++)
				data.putFloat(i * size, java.lang.reflect.Array.getFloat(newArr, i));			
		}
		else if (newArr.getClass().equals(double[].class)) {
			aCode = ACODE_DOUBLE;
			size = 8;
			data = ByteBuffer.allocate(length * size);
			for(int i = 0; i < length; i++)
				data.putDouble(i * size, java.lang.reflect.Array.getDouble(newArr, i));			
		}		
		else if (newArr.getClass().equals(long[].class)) {
			aCode = ACODE_LONG;
			size = 8;
			data = ByteBuffer.allocate(length * size);
			for(int i = 0; i < length; i++)
				data.putLong(i * size, java.lang.reflect.Array.getLong(newArr, i));			
		}		
		else if (newArr.getClass().equals(short[].class)) {
			aCode = ACODE_SHORT;
			size = 2;
			data = ByteBuffer.allocate(length * size);
			for(int i = 0; i < length; i++)
				data.putShort(i * size, java.lang.reflect.Array.getShort(newArr, i));			
		}
		else if (newArr.getClass().equals(char[].class)) {
			aCode = ACODE_CHAR;
			size = 2;
			data = ByteBuffer.allocate(length * size);
			for(int i = 0; i < length; i++)
				data.putChar(i * size, java.lang.reflect.Array.getChar(newArr, i));			
		}		
		else if (newArr.getClass().equals(boolean[].class)) {
			aCode = ACODE_BOOLEAN;
			size = 1;
			data = ByteBuffer.allocate(length * size);
			for(int i = 0; i < length; i++) {
				if (java.lang.reflect.Array.getBoolean(newArr, i))
					data.put(i * size, (byte)1);
				else data.put(i * size, (byte)0);
			}
		}
		else {
			if (log.isDebugEnabled()) {
				//System.out.println("Array type not supported yet");
				log.debug("Array type not supported yet");
			}
			
		}
	}
	
	@Override
	public boolean checkCast(Class cl, int index) {
		return false;
	}

	@Override
	public void getVariable(HardClass currentClass, int index, InterpreterStack stack, References refs)
			throws Exception {
		int i = index * size;
		data.position(i);
		
		switch(size)
		{
			case 1:
				stack.write(data.get(i)); return;
			case 2:
				stack.write(data.getShort(i));return;
			case 4:
				stack.write(data.getInt(i)); return;
			case 8:
				stack.write(data.getLong(i)); return;
		}
		throw new Exception("Error: getVariable - invalid array type size");
	}

	@Override
	public void setVariable(HardClass currentClass, int index, InterpreterStack stack, References refs,
			long value) throws Exception {
		int i = index * size;
		data.position(i);
		
		switch(size)
		{
			case 1:
				data.put(i,(byte)value); return;
			case 2:
				data.putShort(i,(short)value); return;
			case 4:
				data.putInt(i, (int)value); return;
			case 8:
				data.putLong(i, value); return;
		}
		throw new Exception("Error: getVariable - invalid array type size");
	}


	public Object toArray(HashMap<Long, Object> convertedObjects)
		throws Exception
	{
		switch(aCode)
		{
			case ACODE_BYTE: 
				return data.array();
			case ACODE_BOOLEAN:
				Object ret = java.lang.reflect.Array.newInstance(boolean[].class, length);
				for(int i = 0; i < length; i++) {
					if (data.get(i) == 0)
						java.lang.reflect.Array.setBoolean(ret, i, false);
					else java.lang.reflect.Array.setBoolean(ret, i, true);
				}
				return ret;
			case ACODE_SHORT: return data.asShortBuffer().array();	  
			case ACODE_CHAR:
				char[] charRet = new char[length];
				for(int i = 0; i < length; i++)
					charRet[i] = data.getChar(i * size);
				return charRet;
			case ACODE_FLOAT: return data.asFloatBuffer().array();
			case ACODE_INT: return data.asIntBuffer().array();
			case ACODE_DOUBLE: return data.asDoubleBuffer().array();
			case ACODE_LONG: return data.asLongBuffer().array();
			default: throw new Exception("Error: SimpleArray - invalid array type");
		}
	}
	@Override
	public void fromObject(HashMap<Long, Object> convertedObjects, Object newData)
			throws Exception 
	{
		int newLength = java.lang.reflect.Array.getLength(newData);
		if (newLength != length)
			throw new Exception("Array length changed");
		
		switch(aCode)
		{
			case ACODE_BYTE:
				for(int i = 0; i < length; i++)
					data.put(i, java.lang.reflect.Array.getByte(newData, i));
				break;
			case ACODE_BOOLEAN:
				for(int i = 0; i < length; i++)
					if (java.lang.reflect.Array.getBoolean(newData, i))
						data.put(i, (byte)1);
					else data.put(i, (byte)0); 
				break;
			case ACODE_SHORT:
				for(int i = 0; i < length; i++)
					data.putShort(i * size, java.lang.reflect.Array.getShort(newData, i));
				break;
			case ACODE_CHAR:
				for(int i = 0; i < length; i++)
					data.putChar(i * size, java.lang.reflect.Array.getChar(newData, i));
				break;
			case ACODE_FLOAT:
				for(int i = 0; i < length; i++)
					data.putFloat(i * size, java.lang.reflect.Array.getFloat(newData, i));
				break;
			case ACODE_INT:
				for(int i = 0; i < length; i++)
					data.putInt(i * size, java.lang.reflect.Array.getInt(newData, i));
				break;
			case ACODE_DOUBLE:
				for(int i = 0; i < length; i++)
					data.putDouble(i * size, java.lang.reflect.Array.getDouble(newData, i));
				break;
			case ACODE_LONG:
				for(int i = 0; i < length; i++)
					data.putLong(i * size, java.lang.reflect.Array.getLong(newData, i));
				break;
			default: throw new Exception("Error: SimpleArray - invalid array type");
		}
		
	}	

	@Override
	public int getLength() {
		if (data == null)
			return -1;
		return length;
	}
	@Override
	public void getVariable(String name, int length, InterpreterStack stack,
			References refs) throws Exception {
		throw new Exception ("This method shoud be never invoked - something went wrong...");
		
	}

	@Override
	public void setVariable(String name, int length, InterpreterStack stack,
			References refs, long value) throws Exception {
		throw new Exception ("This method shoud be never invoked - something went wrong...");
		
	}

	@Override
	public Reference getSuperReference() throws Exception {
		throw new NotImplementedException(); 
	}

	@Override
	public Class getClassInfo() {
		throw new NotImplementedException();
	}

	@Override
	public boolean isInstanceOf(HardClass currentClass, int classIndex)
			throws Exception {
		throw new Exception();
	}

}
