package san.interpreter.sanint.classLoading;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;

import san.interpreter.sanint.interpreter.InterpreterBytecode;
import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.interpreter.LocalVars;
import san.interpreter.sanint.interpreter.References;
import java.lang.Class;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

public class SoftClass implements san.interpreter.sanint.classLoading.Class {

	ClassManager manager;
	java.lang.Class thisClass;
	
	public SoftClass(String type, ClassManager manager)
		throws ClassNotFoundException
	{
		thisClass = java.lang.Class.forName(type.replace('/', '.'));
		this.manager = manager;
	}
	public SoftClass(java.lang.Class cl, ClassManager manager)
	{
		thisClass = cl;
		this.manager = manager;
	}
	public ClassManager getClassManager()
	{
		return manager;
	}

	public MethodInfo getMethod(String name, String descriptor) {
		return new MethodInfo(name, descriptor);
	}


	public String getName() {
		return thisClass.getName();
	}

	public void runMethod(int index, InterpreterBytecode interpreter,
			References interpreterReferences, InterpreterStack stack,
			LocalVars localVars, boolean staticMethod, boolean virtualMethod) throws Exception {
		throw new Exception("SoftClass.runMethod - cannot interpret methods of soft classes");
	}
	public Constructor getConstructor(MethodInfo method) throws Exception
	{
		Class[] classes = translateDescriptor(method);
		return thisClass.getConstructor(classes);
	}
	public Method getMethod(MethodInfo method) throws Exception
	{
		Class[] classes = translateDescriptor(method);
		java.lang.Class temp = thisClass;
		
		while(temp != null)
		{
			try
			{
				return temp.getDeclaredMethod(method.cachedName, classes);
			}
			catch (NoSuchMethodException e) {}
			if (thisClass.getSuperclass().equals(thisClass))
					break;
			temp = thisClass.getSuperclass();
		}
		throw new NoSuchMethodException();
	}
	
	public Class[] translateDescriptor(MethodInfo method) throws Exception
	{
		ArrayList<Class> classes = new ArrayList<Class>();
		String type = method.cachedDescriptor;
		type = type.substring(type.indexOf('(')+1, type.lastIndexOf(')'));
							
		int i = 0;
		while( i < type.length())
		{
			switch (type.charAt(i))
			{
			case 'I': classes.add(int.class); break;
			case 'F': classes.add(float.class); break;
			case 'D': classes.add(double.class); break;
			case 'J': classes.add(long.class); break;
			case 'Z': classes.add(boolean.class); break;
			case 'B': classes.add(byte.class); break;
			case 'C': classes.add(char.class); break;
			case 'S': classes.add(short.class); break;
			case 'L': 
				int from = i + 1;
				int to = from;
				for(to = from; to < type.length() && type.charAt(to) != ';'; to++);
				to--;
				Class cl = Class.forName(type.substring(from, to + 1).replace('/', '.'));
				classes.add(cl);
				i = to + 1;
				break;
			case '[':
				i = getArrayClass(type, i, classes) - 1;
				break;
			default: throw new Exception("SoftClass.translateDescriptor - unknown type");
			}

			i++;
		}
		return (Class[])classes.toArray(new Class[] {});
	}
	
	public int getArrayClass(String descriptor, int index, ArrayList<Class> classes)
		throws Exception
	{
		while(descriptor.charAt(index) == '[')
			index++;
		switch (descriptor.charAt(index))
		{
		case 'I': classes.add(int[].class); break;
		case 'F': classes.add(float[].class); break;
		case 'D': classes.add(double[].class); break;
		case 'J': classes.add(long[].class); break;
		case 'Z': classes.add(boolean[].class); break;
		case 'B': classes.add(byte[].class); break;
		case 'C': classes.add(char[].class); break;
		case 'S': classes.add(short[].class); break;
		case 'L': throw new Exception("SoftClass.getArrayClass - reference arrays not supported" + 
									" as parameters of soft methods.");
		default: throw new Exception("SoftClass.getArrayClass - unknown type.");
		}
		return index + 1;
	}
	public boolean canBeCasted(String type) {
		if (thisClass.getName().equals(type))
			return true;
		java.lang.Class superClass = thisClass.getSuperclass();
		while (superClass != null)
		{
			if(superClass.getCanonicalName().equals(type))
				return true;
			superClass = superClass.getSuperclass();
		}
		return false;
	}
	public void getStaticVariable(String name, InterpreterStack stack,
			References references) throws Exception {
		Field field = thisClass.getField(name);
		Type fieldType = field.getGenericType();
		
		if (fieldType.equals(int.class))
			stack.write(field.getInt(null));
		else if (fieldType.equals(float.class))
				stack.write(field.getFloat(null));
		else if (fieldType.equals(double.class))
			stack.write(field.getDouble(null));
		else if (fieldType.equals(long.class))
			stack.write(field.getLong(null));
		else if (fieldType.equals(boolean.class))
			if (field.getBoolean(null))
				stack.write(1);
			else stack.write(0);
		else if (fieldType.equals(byte.class))
			stack.write(field.getByte(null));
		else if (fieldType.equals(char.class))
			stack.write(field.getChar(null));
		else if (fieldType.equals(short.class))
			stack.write(field.getShort(null));
		else {
			Object value = field.get(null);
			stack.write(
					references.putReference(
							new SoftReference(value, manager)));
		}
		return;
	}
	public void setStaticVariable(String name, InterpreterStack stack, References references)
		throws Exception
	{
		throw new NotImplementedException();
		
	}

}
