package san.interpreter.sanint.classLoading;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;

import com.sun.org.apache.bcel.internal.classfile.ConstantClass;
import com.sun.org.apache.bcel.internal.classfile.ConstantFieldref;
import com.sun.org.apache.bcel.internal.classfile.ConstantNameAndType;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import san.interpreter.sanint.interpreter.InterpreterBytecode;
import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.interpreter.LocalVars;
import san.interpreter.sanint.interpreter.References;

public class SoftReference extends Reference {

	SoftClass thisClass;
	Object reference;
	

	public SoftReference(SoftClass cl)
	{
		thisClass = cl;
	}
	
	public SoftReference(Object reference, ClassManager manager)
	{
		this.reference = reference;
		if (reference != null)
			thisClass = new SoftClass(reference.getClass(), manager);
	}
	
	Object getReference()
	{
		return reference;
	}
	
	@Override
	public boolean checkCast(Class cl, int index) {
		return false;
	}

	@Override
	public void getVariable(HardClass currentClass, int index, InterpreterStack stack, References refs)
			throws Exception {
		ConstantFieldref fieldRef = (ConstantFieldref)currentClass.cl.getConstantPool().getConstant(index);
		String className = fieldRef.getClass(currentClass.cl.getConstantPool());
		ConstantNameAndType nameAndType = (ConstantNameAndType)
					currentClass.cl.getConstantPool().getConstant(fieldRef.getNameAndTypeIndex());
		String name = nameAndType.getName(currentClass.cl.getConstantPool());
		String type = nameAndType.getSignature(currentClass.cl.getConstantPool());
		java.lang.Class cl = java.lang.Class.forName(className);
		Field field = cl.getDeclaredField(name);
		field.setAccessible(true);
		
		switch (type.charAt(0))
		{
		case 'I':
			stack.write(field.getInt(reference));
			break;
		case 'F':
			stack.write(field.getFloat(reference));
			break;
		case 'D': 
			stack.write(field.getDouble(reference));
			break;
		case 'J':
			stack.write(field.getLong(reference));
			break;
		case 'Z': 
			if (field.getBoolean(reference))
				stack.write((int)1);
			else stack.write((int)0); 
			break;
		case 'B': 
			stack.write(field.getByte(reference));
			break;
		case 'C': 
			stack.write(field.getChar(reference));
			break;
		case 'S': 
			stack.write(field.getShort(reference));
			break;
		case 'L': 
			ReferenceConverter conv = new ReferenceConverter();
			Reference ref = conv.toHardReference(field.get(reference), thisClass.getClassManager() );
			int newReferenceIndex = refs.putReference(ref);
			stack.write(newReferenceIndex);
			break;
		case '[':
			Object a = field.get(reference);
			if (type.charAt(1) == 'L')
				throw new NotImplementedException();
			else
			{
				SimpleArray arr = new SimpleArray(a);
				stack.write(refs.putReference(arr));
			}
			break;
		default:
			throw new Exception("SoftReference.createParameters - not supported parameter type.");
		
		}
		
	}

	@Override
	public void setVariable(HardClass currentClass, int index, InterpreterStack stack, References refs,
			long value) throws Exception {
		throw new NotImplementedException();

	}


	@Override
	public void getVariable(String name, int length, InterpreterStack stack,
			References refs) throws Exception {
		throw new NotImplementedException();
	}


	@Override
	public void setVariable(String name, int length, InterpreterStack stack,
			References refs, long value) throws Exception {
		throw new NotImplementedException();
		
	}

	public void runMethod(MethodInfo methodInfo, 
			References references,
			InterpreterStack stack,
			ClassManager manager)
		throws Exception
	{
		if (reference == null && thisClass == null)
			throw new NullPointerException();
		ArrayList inputParams = new ArrayList();
		java.lang.Class[] paramsClasses = thisClass.translateDescriptor(methodInfo);
		Object[] params = createParameters(methodInfo.cachedDescriptor, stack, references, paramsClasses, inputParams);
		
		stack.setStackPointer(stack.stackPointer() - 4);
		
		if (reference == null)
		{
			//mel by se poustet konstruktor
			Constructor constructor = thisClass.getConstructor(methodInfo);
			reference = constructor.newInstance(params);
			return;
		}
		
		Method method = thisClass.getMethod(methodInfo);
		method.setAccessible(true);
		Object ret = method.invoke(reference, params);
		SoftReference.saveReturnValue(ret, stack, references, methodInfo.cachedDescriptor, manager);
		updateParameters(params, paramsClasses, inputParams, references, false);		


	}
	public static void runStaticMethod(SoftClass cl,
						MethodInfo methodInfo, 
						References references,
						InterpreterStack stack,
						ClassManager manager)	throws Exception
	{
		ArrayList inputParams = new ArrayList();
		Method method = cl.getMethod(methodInfo);		
		Object[] params = createParameters(methodInfo.cachedDescriptor, stack, references,
										   method.getParameterTypes(), inputParams);

		
		Object ret = method.invoke(null, params);
		SoftReference.saveReturnValue(ret, stack, references, methodInfo.cachedDescriptor, manager);
	}
	
	
	protected static Object[] createParameters(String descriptor, InterpreterStack stack,
										References references, java.lang.Class[] types, ArrayList inputParams)
		throws Exception
	{
		ArrayList<Object> params = new ArrayList<Object>();
		
		for(int i = types.length - 1; i >= 0; i--)
		{
			if (types[i].isPrimitive())
			{
				if (types[i].equals(int.class))
					params.add(0, stack.readInt());
				else if (types[i].equals(float.class))
					params.add(0, stack.readFloat());
				else if (types[i].equals(double.class))
					params.add(0, stack.readDouble());
				else if (types[i].equals(long.class))
					params.add(0, stack.readLong());
				else if (types[i].equals(boolean.class))
					params.add(0, stack.readByte() != 0);
				else if (types[i].equals(byte.class))
					params.add(0, stack.readByte());
				else if (types[i].equals(char.class))
					params.add(0, (char)stack.readShort());
				else if (types[i].equals(short.class))
					params.add(0, stack.readShort());
				else throw new Exception("unknown primitive type");
				inputParams.add(0, null);
			}
			else if (types[i].isArray())
			{
				Reference arrayRef = references.getReference(stack.readRef());
				if (!( arrayRef instanceof Array))
					System.out.println();
				Array a = (Array)arrayRef;
				ReferenceConverter convArr = new ReferenceConverter();
				params.add(0, convArr.toJavaObject(a, null));
				inputParams.add(0, a);
			}
			else
			{
				Reference r = references.getReference(stack.readInt());

				ReferenceConverter convRef = new ReferenceConverter();
				params.add(0, convRef.toJavaObject(r, null));
				inputParams.add(0, r);
			}
		}
		
		
		/*
		String type = descriptor;
		type = type.substring(type.indexOf('(')+1, type.lastIndexOf(')'));
		int i = 0;
		while (i < type.length())
		{
			switch (type.charAt(i))
			{
			case 'I': params.add(stack.readInt()); break;
			case 'F': params.add(stack.readFloat()); break;
			case 'D': params.add(stack.readDouble()); break;
			case 'J': params.add(stack.readLong()); break;
			case 'Z': params.add(stack.readByte() != 0); break;
			case 'B': params.add(stack.readByte()); break;
			case 'C': params.add((char)stack.readInt()); break;
			case 'S': params.add(stack.readShort()); break;
			case 'L': 
				int to;
				for(to = i + 1; to < type.length() && type.charAt(to) != ';'; to++);
				Reference r = references.getReference(stack.readInt());
				
				ReferenceConverter convRef = new ReferenceConverter();
				params.add(convRef.toJavaObject(r, null));
				i = to;
				break;
			case '[':
				while(type.charAt(i) ==  '[')
					i++;
				if (type.charAt(i) == 'L')
					while(type.charAt(i) != ';')
						i++;
				Reference arrayRef = references.getReference(stack.readRef());
				if (!( arrayRef instanceof Array))
					System.out.println();
				Array a = (Array)arrayRef;
				ReferenceConverter convArr = new ReferenceConverter();
				params.add(convArr.toJavaObject(a, null));
				break;
			default:
				throw new Exception("SoftReference.createParameters - not supported parameter type.");
			
			}
			i++;
		}	*/
		return params.toArray();
	}
	protected static void saveReturnValue(Object ret, InterpreterStack stack, References references, String descriptor, ClassManager manager)
	{
		String returnValueStr = descriptor.substring(descriptor.lastIndexOf(')') + 1);
		switch (returnValueStr.charAt(0))
		{
		case 'I': stack.write(((Integer)ret).intValue()); break;
		case 'F': stack.write(((Float)ret).floatValue()); break;
		case 'D': stack.write(((Double)ret).doubleValue()); break;
		case 'J': stack.write(((Long)ret).longValue()); break;
		case 'Z': 	if(((Boolean)ret).booleanValue())
						stack.write(1);
					else stack.write(0);
					break;
		case 'B': stack.write(((Byte)ret).byteValue()); break;
		case 'C': stack.write(((Character)ret).charValue()); break;
		case 'S': stack.write(((Short)ret).shortValue()); break;
		case 'L': 
			stack.write(references.putReference(new SoftReference(ret, manager)));
			break;
		case '[':
			stack.write(references.putReference(new SimpleArray(ret)));
			break;
		}
	}

	private void updateParameters(Object[] args, java.lang.Class[] types, ArrayList inputParams, References refs, boolean isStatic)
	throws Exception
	{
	for(int i = 0; i < args.length; i++)
	{
		if (!types[i].isPrimitive())
		{
			Reference ref = (Reference)inputParams.get(i);
			Object obj = args[i];
			ref.fromObject(null, obj);
		}
	}
}
	
	
	public boolean isNull()
	{
		return reference == null && thisClass == null;
	}

	@Override
	public Reference getSuperReference() throws Exception {
		return this;
	}

	@Override
	public Class getClassInfo() {
		return thisClass;
	}

	@Override
	public void fromObject(HashMap<Long, Object> convertedObjects,
			Object newData) throws Exception 
	{
		if (thisClass.getName().equals(newData.getClass().getName()))
			reference = newData;
		else throw new Exception("Types does not match");
	}

	@Override
	public boolean isInstanceOf(HardClass currentClass, int classIndex)
			throws Exception {
		ConstantClass clConst =  (ConstantClass)currentClass.getConstants().getConstant(classIndex);
		String className = clConst.getBytes(currentClass.getConstants());		
		return thisClass.canBeCasted(className);
	}
	
}
