/*************************************************************************
 *                           Smart Active Node                           * 
 *                      copyright (C) 2007 KIV-FAV-ZCU                   *
 *************************************************************************
 * The licence for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.interpreter.sanint.test;

import java.io.Serializable;
import java.util.Random;

/**
 * <b>Title:</b> Smart Active Node <b>Description:</b> This class represent
 * the GUID identifier of node.
 * @author Jan Boháč, Michal Rejda
 * @since 6.11.2006
 * @version 1.1
 */
public class Guid implements Comparable<Guid>, Serializable {

	/**
	 * Count of Guids bytes. It's set to 16 bytes = 128 bites.
	 */
	public static final int LENGHT = 16;
	private byte[] guid; // the guid

	private Guid() {
		this.guid = null;
	}

	public Guid(byte[] guid) {
		this.guid = guid;
	}

	public Guid(Guid guid) {
		this.guid = new byte[guid.guid.length];
		// making copy of the guid
		for (int i = 0; i < guid.guid.length; i++) {
			this.guid[i] = guid.guid[i];
		}
	}

	/**
	 * Return random generated guid.
	 */
	public static Guid randomGuid() {
		Random ran = new Random();
		byte[] guid = new byte[LENGHT];
		ran.nextBytes(guid);
		return new Guid(guid);
	}

	/**
	 * Return the Guid as byte array
	 */
	public byte[] toByte() {
		return guid;
	}

	/**
	 * Convert Guids string formated as: 00000000-0000-0000-0000-000000000000 to
	 * Guid.
	 * @param s Guids string formated as: 00000000-0000-0000-0000-000000000000
	 * @return Guid
	 */
	public static Guid parseGuid(String s) {
		if (s == null)
			return null;
		if (s.length() != (LENGHT * 2 + 4))
			throw new NumberFormatException("Bad Guid string representation.");
		// construct bytes of guid
		byte[] bytes = new byte[LENGHT];
		String hex;
		int offset = 0;
		for (int i = 0; i < bytes.length; i++) {
			// converting hex string to byte
			bytes[i] = (byte) Integer.parseInt(s.substring(2 * i + offset, 2
					* i + offset + 2), 16);
			// skipping dashes
			if (i == 3 || i == 5 || i == 7 || i == 9) {
				offset++;
			}
		}
		// creating guid
		Guid g = new Guid();
		g.guid = bytes;
		return g;
	}

	public int compareTo(Guid arg0) {
		if (this.guid.length != arg0.guid.length) {
			return this.guid.length - arg0.guid.length;
		} else
			for (int i = 0; i < this.guid.length; i++) {
				if (this.guid[i] != arg0.guid[i]) {
					return (this.guid[i] - arg0.guid[i]) * i;
				}
			}
		return 0;
	}

	public boolean equals(Object arg0) {
		if (arg0 instanceof Guid) {
			Guid guid = (Guid) arg0;
			if (this.guid.length != guid.guid.length) {
				return false;
			} else {
				for (int i = 0; i < this.guid.length; i++) {
					if (this.guid[i] != guid.guid[i]) {
						return false;
					}
				}
				return true;
			}
		} else {
			return false;
		}
	}

	/**
	 * Get guids string formated as: 00000000-0000-0000-0000-000000000000
	 */
	public String toString() {
		char[] val = new char[2 * guid.length + 4];
		String hex = "0123456789ABCDEF";
		int offset = 0;
		int b;
		for (int i = 0; i < guid.length; i++) {
			b = guid[i] & 0xff;
			val[2 * i + offset] = hex.charAt(b >>> 4);
			val[2 * i + offset + 1] = hex.charAt(b & 15);
			if (i == 3 || i == 5 || i == 7 || i == 9) {
				offset++;
				val[2 * i + offset + 1] = '-';
			}
		}
		return String.valueOf(val);
	}

	public int hashCode() {
		int hash = 0;
		for (int i = 0; i < guid.length; i++) {
			hash = hash * 3 + guid[i];
		}
		return hash;
	}
}
