/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.localization;

import java.text.MessageFormat;
import java.util.ResourceBundle;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class provides localization.<br>
 * @author Michal Rejda
 * @since 22.11.2007
 * @version 1.0
 */
public class Localization {

	public static String LOCALIZATION_RESOURCE = "localization";
	private static final Localization localization = new Localization();
	private ResourceBundle resourceBundle;
	private MessageFormat formatter = new MessageFormat("");

	private Localization() {
		resourceBundle = ResourceBundle.getBundle(LOCALIZATION_RESOURCE);
	}

	/**
	 * Gets a string for the given key from this resource bundle or one of its
	 * parents.
	 * @param key the key for the desired string
	 * @return the string for the given key
	 * @throws NullPointerException if key is null
	 * @throws MissingResourceException if no object for the given key can be
	 *             found
	 * @throws ClassCastException if the object found for the given key is not a
	 *             string
	 */
	public String getString(String key) {
		return resourceBundle.getString(key);
	}
	
	/**
	 * Gets a string for the given key from this resource bundle or one of its
	 * parents.
	 * @param key the key for the desired string
	 * @param values values in the string
	 * @return the string for the given key
	 * @throws NullPointerException if key is null
	 * @throws MissingResourceException if no object for the given key can be
	 *             found
	 * @throws ClassCastException if the object found for the given key is not a
	 *             string
	 */
	public String getString(String key,Object[] values) {
		formatter.applyPattern(resourceBundle.getString(key));
		return formatter.format(values);
	}
	
	/**
	 * @return the localization
	 */
	public static Localization getLocalization() {
		return localization;
	}
}
