/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.routing;

import java.io.Serializable;

import san.ident.NetIdentifier;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> RoutingRecord.java<br>
 * @author Michal Rejda
 * @since 30.10.2007
 * @version 1.0
 */
public class RoutingRecord implements Serializable {

	/**
	 * <b>Title:</b> Smart Active Node<br>
	 * <b>Description:</b> RoutingRecord.java<br>
	 * @author Michal Rejda
	 * @since 31.10.2007
	 * @version 1.0
	 */
	public enum Type {
		STATIC, DYNAMIC
	}

	/**
	 * @uml.property name="description"
	 */
	private String description = "";
	/**
	 * @uml.property name="metric"
	 */
	private int metric;
	/**
	 * @uml.property name="destination"
	 */
	private NetIdentifier destination;
	/**
	 * @uml.property name="gateway"
	 */
	private NetIdentifier gateway;
	/**
	 * @uml.property name="netInterface"
	 */
	private NetIdentifier netInterface;
	/**
	 * @uml.property name="age"
	 */
	private long age;
	/**
	 * @uml.property name="type"
	 */
	private RoutingRecord.Type type;

	/**
	 * Return true if destination and gateway in compared object are identical.
	 */
	public boolean equals(Object obj) {
		if (obj instanceof RoutingRecord) {
			RoutingRecord rr = (RoutingRecord) obj;
			return rr.getDescription().equals(this.destination)
					&& rr.getGateway().equals(this.gateway);
		}
		return false;
	}

	/**
	 * Getter of the property <tt>description</tt>
	 * @return Returns the description.
	 * @uml.property name="description"
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Setter of the property <tt>description</tt>
	 * @param description The description to set.
	 * @uml.property name="description"
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Getter of the property <tt>metric</tt>
	 * @return Returns the metric.
	 * @uml.property name="metric"
	 */
	public int getMetric() {
		return metric;
	}

	/**
	 * Setter of the property <tt>metric</tt>
	 * @param metric The metric to set.
	 * @uml.property name="metric"
	 */
	public void setMetric(int metric) {
		this.metric = metric;
	}

	/**
	 * Getter of the property <tt>destination</tt>
	 * @return Returns the destination.
	 * @uml.property name="destination"
	 */
	public NetIdentifier getDestination() {
		return destination;
	}

	/**
	 * Setter of the property <tt>destination</tt>
	 * @param destination The destination to set.
	 * @uml.property name="destination"
	 */
	public void setDestination(NetIdentifier destination) {
		this.destination = destination;
	}

	/**
	 * Getter of the property <tt>gateway</tt>
	 * @return Returns the gateway.
	 * @uml.property name="gateway"
	 */
	public NetIdentifier getGateway() {
		return gateway;
	}

	/**
	 * Setter of the property <tt>gateway</tt>
	 * @param gateway The gateway to set.
	 * @uml.property name="gateway"
	 */
	public void setGateway(NetIdentifier gateway) {
		this.gateway = gateway;
	}

	/**
	 * Getter of the property <tt>netInterface</tt>
	 * @return Returns the netInterface.
	 * @uml.property name="netInterface"
	 */
	public NetIdentifier getNetInterface() {
		return netInterface;
	}

	/**
	 * Setter of the property <tt>netInterface</tt>
	 * @param netInterface The netInterface to set.
	 * @uml.property name="netInterface"
	 */
	public void setNetInterface(NetIdentifier netInterface) {
		this.netInterface = netInterface;
	}

	/**
	 * Getter of the property <tt>age</tt>
	 * @return Returns the age.
	 * @uml.property name="age"
	 */
	public long getAge() {
		return age;
	}

	/**
	 * Setter of the property <tt>age</tt>
	 * @param age The age to set.
	 * @uml.property name="age"
	 */
	public void setAge(long age) {
		this.age = age;
	}

	/**
	 * Getter of the property <tt>type</tt>
	 * @return Returns the type.
	 * @uml.property name="type"
	 */
	public RoutingRecord.Type getType() {
		return type;
	}

	/**
	 * Setter of the property <tt>type</tt>
	 * @param type The type to set.
	 * @uml.property name="type"
	 */
	public void setType(RoutingRecord.Type type) {
		this.type = type;
	}
}
