/* 
 * File:   ii.c
 * Author: kubajz
 *
 * Created on 18. červenec 2009, 19:32
 */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <string.h>
#include <getopt.h>
#include <ctype.h>
#include <stdarg.h>
#include "injector.h"
#include "interceptor.h"
#include "ii.h"

#define MAX_LINE_LENGTH 256

extern void connector(void);

char *server, *gatewaynode, *gatewaynetwork, *application;
int port, verbose = 0, nfqueue, connector_only = 0;

/**
 *trims string's blanks from beginning and end. It does allocate memory,
 * it frees string passed as argument
 * returns newly allocated string
 */
char *strtrim(char *str) {
    int i = 0, j = 0;
    char *retval;

    for (i = 0; i < strlen(str) && !isalnum(str[i]); i++);

    for (j = strlen(str) - 1; j > 0 && !isalnum(str[j]); j--);

    retval = calloc(j - i + 2, sizeof(char));
    memcpy(retval, str + i, j - i + 1);

    free(str);

    return retval;
}

void read_configuration(const char *conf_filename) {
    char line[MAX_LINE_LENGTH];
    FILE *fp = fopen(conf_filename, "r");
    int options = 0; //number of parsed options

    if (fp == NULL) {
        printe("Unable to open configuration file %s\n", conf_filename);
        exit(EXIT_FAILURE);
    }

    memset(line, 0, MAX_LINE_LENGTH);

    while (fgets(line, MAX_LINE_LENGTH, fp) != NULL) {
        char *option, *value, *hlp;
        int eq = 0;

        if (strchr(line, '=') == NULL) {
            continue;
        }

        option = strdup(line);
        eq = strchr(option, '=') - option;
        option[eq] = '\0';
        option = strtrim(option);

        value = strdup(strchr(line, '=') + 1);
        value = strtrim(value);

        if (strcmp(option, "host") == 0) {
            options = options | 0x1;
            server = value;
        } else if (strcmp(option, "port") == 0) {
            options = options | (0x1 << 1);
            port = atoi(value);
        } else if (strcmp(option, "gatewaynode") == 0) {
            options = options | (0x1 << 2);
            gatewaynode = value;
        } else if (strcmp(option, "gatewaynetwork") == 0) {
            options = options | (0x1 << 3);
            gatewaynetwork = value;
        } else if (strcmp(option, "application") == 0) {
            options = options | (0x1 << 4);
            application = value;
        } else if (strcmp(option, "nfqueue") == 0) {
            options = options | (0x1 << 5);
            nfqueue = atoi(value);
        } else {
            free(value);
            value = NULL;
        }

        free(option);
        option = NULL;
    }

    if (options != 0x3F) {
         printe( "Not all required configuration read from config file!");

        if ((options & 0x1) == 0) {
            printe("host not set");
        }

        if ((options & (0x1<<1)) == 0) {
            printe("port not set");
        }

        if ((options & (0x1<<2)) == 0) {
            printe("gatewaynode not set");
        }

        if ((options & (0x1<<3)) == 0) {
            printe("gatewaynetwork not set");
        }

        if ((options & (0x1<<4)) == 0) {
            printe("application not set");
        }

        if ((options & (0x1<<5)) == 0) {
            printe("nfqueue not set");
        }
        
        exit(EXIT_FAILURE);
    }
}

void print_usage(const char *binname) {
    printf("Usage: %s [-v] -c configuration_file\n\t-v makes output more verbose\n",
            binname);
}

/*
 * 
 */
int main(int argc, char** argv) {
    pthread_t interceptor_thread, injector_thread;
    int retval, sockfd, opt;
    char *conf_file = NULL;
    struct sockaddr_in rem_addr;

    while ((opt = getopt(argc, argv, "Cvc:")) != -1) {
        switch (opt) {
            case 'v':
                verbose = 1;
                break;
            case 'c':
                conf_file = strdup(optarg);
                break;
            case 'C':
                connector_only = 1;
                break;
            default: /* '?' */
                print_usage(argv[0]);
                exit(EXIT_FAILURE);
        }
    }

    if (connector_only == 1) {
        connector();
        exit(EXIT_SUCCESS);
    }

    if (conf_file == NULL) {
        print_usage(argv[0]);
        exit(EXIT_FAILURE);
    }

    read_configuration(conf_file);

    
    printd("succesfully read configuration server=%s:%d@%s through %s-%s IPv4gw %s call application %s and listen in nfqueue %d\n",
            server, port, interface, gatewaynode, gatewaynetwork,
            ipgateway, application, nfqueue);
    
    /* try to allocate socket for communication */
    if ((sockfd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
        printe( "Unable to create socket!\n");
        exit(EXIT_FAILURE);
    }

    /* prepare connection details */
    rem_addr.sin_family = AF_INET; /* IPv4 connection */
    rem_addr.sin_port = htons(port); /* server listen port */
    retval = inet_aton(server, &(rem_addr.sin_addr)); /* connect to host */

    /* connect to server */
    if ((retval = connect(sockfd, (struct sockaddr *) & rem_addr, sizeof (rem_addr))) == -1) {
        printe( "Unable to connect to server!\n");
        exit(EXIT_FAILURE);
    };

    retval = pthread_create(&interceptor_thread, NULL, interceptor_run, &sockfd);
    retval = pthread_create(&injector_thread, NULL, injector_run, &sockfd);

    pthread_join(interceptor_thread, NULL);
    pthread_join(injector_thread, NULL);

    close(sockfd);

    return (EXIT_SUCCESS);
}

