#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>

#include "ii.h"
#include "injector.h"

int injector_sockfd, ifindex;

void *injector_run(void *param) {
    injector_sockfd = *(int *) param;
    P_PACKET p;
    int sock;

    printd("injector thread running");

    /*
     *   Create injection socket. We want to inject whole IPv4 packet including
     *   header hence we use SOCK_RAW and IPPROTO_RAW, which tells system not
     *   to include automatically generated header
     */
    if ((sock = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) == -1) {
        printe("Cannot create injection socket");
        return;
    }

    while ((p = receive_packet()) != NULL) {
        inject_packet(p, sock);
    }
}

char *get_destip(P_PACKET p) {
    int ipaddr_n;
    struct in_addr ina;
    char *retstr;

    /* dst address is bytes 16,17,18,19 according to RFC */
    memcpy(&ipaddr_n, (p->payload) + 16, 4);

    ina.s_addr = ipaddr_n;

    retstr = inet_ntoa(ina);

    printd("destination IP %s read from packet", retstr);

    return retstr;
}

/** frees allocated memory by p */
void recycle_packet(P_PACKET p) {
    if (p) {

        if (p->params) { //free all parameters
            for (p->param_count--; p->param_count >= 0; p->param_count--) {
                if (p->params[p->param_count]) {
                    free(p->params[p->param_count]);
                    p->params[p->param_count] = NULL;
                }
            }
            free(p->params);
        }

        if (p->appname) { //free appname
            free(p->appname);
            p->appname = NULL;
        }

        if (p->payload) { //free payload
            free(p->payload);
            p->payload = NULL;
        }

        free(p);
    }
}

/* Inject paket back to network stack */
void inject_packet(P_PACKET p, int sock) {
    struct sockaddr_in sin;
    int ssize;

    /* inject packet only if it is designed by our active application tunneler */
    if (p->appname != NULL && strcmp(p->appname, application) == 0) {

        sin.sin_family = AF_INET; //IPv4 packet
        sin.sin_port = htons(0); //no need to provide port
        inet_aton(get_destip(p), &(sin.sin_addr)); //provide for routing decision

        if ((ssize = sendto(sock,
                p->payload, //data packet including header
                p->payload_size, //data packet size in bytes
                0, //flags such as OOB, DONTROUTE etc.
                (struct sockaddr *) & sin,
                sizeof (sin))) < 0) {
            printe("Sendto failed");
        } else if (ssize != p->payload_size) {
            printe("Not all bytes of packet sent. Sent %d/%d Bytes.", ssize, p->payload_size);
        }

        printd("Sent %d Byte packet", ssize);
    } else {
        printd("Data sent from application different from %s", application);
    }

    recycle_packet(p); //free allocated memory
}

P_PACKET receive_packet() {
    P_PACKET p = calloc(sizeof (PACKET), 1);

    p->appname = read_string();

    p->param_count = read_short();

    if (p->param_count > 0) {
        int i;

        p->params = calloc(p->param_count, sizeof (char *));

        for (i = 0; i < p->param_count; i++) {
            (p->params)[i] = read_string();
        }
    }

    p->payload_size = read_short();

    p->payload = read_bytes(p->payload_size);

    if (p->appname == NULL || p->param_count < 0 || p->payload_size < 0 || p->payload == NULL) {
        recycle_packet(p);
        return NULL;
    }

    return p;
}

char *read_bytes(int size) {
    int recvbytes = 0;
    char *bytes;

    if (size <= 0) {
        return NULL;
    }

    bytes = calloc(size, sizeof (char));
    recvbytes = recv(injector_sockfd, bytes, size, MSG_WAITALL);

    if (size != recvbytes) {
        printd("bytes mismatch");
        free(bytes);
        return NULL;
    }

    return bytes;
}

/* reads short from socket */
int read_short() {
    int retval;
    int recvbytes = 0;

    recvbytes = recv(injector_sockfd, &retval, 2, MSG_WAITALL);
    if (recvbytes != 2) {
        printd("short mismatch")
        return -1;
    }

    retval = ntohs(retval);

    return (retval >= 0) ? retval : -1;
}

/* reads string from socket */
char *read_string() {
    int len;
    int recvbytes;
    char *str;

    len = read_short();
    if (len < 0) {
        return NULL;
    }

    str = calloc(len + 1, sizeof (char)); //one more for \0

    recvbytes = recv(injector_sockfd, str, len, MSG_WAITALL);
    if (recvbytes != len) {
        printd("string mismatch");
        free(str);
        return NULL;
    }

    str[len] = '\0';

    return str;
}
