#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <linux/types.h>
#include <linux/netfilter.h>		/* for NF_ACCEPT, NF_STOLEN etc. */
#include "interceptor.h"
#include "ii.h"

#include <libnetfilter_queue/libnetfilter_queue.h>

#define SHORTBYTES  2
#define MAX_IP_PACKET_LEN 4096

int pktcnt = 0;
int interceptor_sockfd; /* in fact injector and interceptor use the same socket*/

void *interceptor_run(void *param) {
    interceptor_sockfd = *(int *) param;

    struct nfq_handle *h;
    struct nfq_q_handle *qh;
    struct nfnl_handle *nh;
    int fd;
    int rv;
    char buf[MAX_IP_PACKET_LEN] __attribute__((aligned));

    printd("interceptor thread running\n");

    printd("opening library handle\n");
    h = nfq_open();
    if (!h) {
        printe( "error during nfq_open()\n");
        exit(1);
    }

    printd("unbinding existing nf_queue handler for AF_INET (if any)\n");
    if (nfq_unbind_pf(h, AF_INET) < 0) {
        printe( "error during nfq_unbind_pf()\n");
        exit(1);
    }

    printd("binding nfnetlink_queue as nf_queue handler for AF_INET\n");
    if (nfq_bind_pf(h, AF_INET) < 0) {
        printe( "error during nfq_bind_pf()\n");
        exit(1);
    }

    printd("binding this socket to queue '%d'", nfqueue);
    qh = nfq_create_queue(h, nfqueue, &cb, NULL);
    if (!qh) {
        printe( "error during connection to NFQUEUE %d. Maybe some other application is using it or the queue does not exist.", nfqueue);
        exit(1);
    }

    printd("setting copy_packet mode\n");
    if (nfq_set_mode(qh, NFQNL_COPY_PACKET, 0xffff) < 0) {
        printe( "can't set packet_copy mode\n");
        exit(1);
    }

    nh = nfq_nfnlh(h);
    fd = nfnl_fd(nh);

    while ((rv = recv(fd, buf, sizeof (buf), 0)) > 0) {
        printd("pkt received\n");
        nfq_handle_packet(h, buf, rv);
    }

    printd("unbinding from queue 0\n");
    nfq_destroy_queue(qh);

    printd("closing library handle\n");
    nfq_close(h);

}

static u_int32_t print_pkt(struct nfq_data *tb) {
    int id = 0;
    struct nfqnl_msg_packet_hdr *ph;
    u_int32_t mark, ifi;
    int ret;
    char *data;

    ph = nfq_get_msg_packet_hdr(tb);
    if (ph) {
        id = ntohl(ph->packet_id);
        printd("hw_protocol=0x%04x hook=%u id=%u ",
                ntohs(ph->hw_protocol), ph->hook, id);
    }

    mark = nfq_get_nfmark(tb);
    if (mark)
        printd("mark=%u ", mark);

    ifi = nfq_get_indev(tb);
    if (ifi)
        printd("indev=%u ", ifi);

    ifi = nfq_get_outdev(tb);
    if (ifi)
        printd("outdev=%u ", ifi);

    ret = nfq_get_payload(tb, &data);
    if (ret >= 0) {
        printd("payload_len=%d ", ret);
        send_packet(data, ret);
    }

    return id;
}

/*
 * sends packet to java ii with numbers in network order
 */
void send_packet(char *payload, int payload_len) {
    char buffer[MAX_IP_PACKET_LEN]; //sending buffer
    char *buff_point;
    int appnamelen_n = htons(strlen(application));
    int paramcount_n = htons(2);
    int gatewaynode_n = htons(strlen(gatewaynode));
    int gatewaynet_n = htons(strlen(gatewaynetwork));

    int payload_len_n = htons(payload_len);
    int wrbytes = 0;

    buff_point = buffer;
    
    

    memcpy(buff_point, &appnamelen_n, SHORTBYTES);
    buff_point += SHORTBYTES;
    
    memcpy(buff_point, application, strlen(application));
    buff_point += strlen(application);

    memcpy(buff_point, &paramcount_n, SHORTBYTES);
    buff_point += SHORTBYTES;
    
    memcpy(buff_point, &gatewaynode_n, SHORTBYTES); // param 1
    buff_point += SHORTBYTES;
    memcpy(buff_point, gatewaynode, strlen(gatewaynode));
    buff_point += strlen(gatewaynode);

    memcpy(buff_point, &gatewaynet_n, SHORTBYTES); // param 2
    buff_point += SHORTBYTES;
    memcpy(buff_point, gatewaynetwork, strlen(gatewaynetwork));
    buff_point += strlen(gatewaynetwork);

    memcpy(buff_point, &payload_len_n, SHORTBYTES);
    buff_point += SHORTBYTES;

    memcpy(buff_point, payload, payload_len);
    buff_point += payload_len;

    wrbytes = write(interceptor_sockfd, buffer, buff_point-buffer); //write buffer at once
    
    if (wrbytes < 0 || wrbytes < (buff_point-buffer)) {
	printe("Unable to write to SAN socket");
    }
}

/**
 *callback function called for queued packet
 */
static int cb(struct nfq_q_handle *qh, struct nfgenmsg *nfmsg,
        struct nfq_data *nfa, void *data) {
    printd("entering callback");
    u_int32_t id = print_pkt(nfa);
    return nfq_set_verdict(qh, id, NF_STOLEN, 0, NULL);
}